﻿Imports System.Threading.Tasks
Imports Windows.ApplicationModel.Core
Imports Windows.UI.Core
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls
Imports Windows.UI.Xaml.Media

Namespace SimpleOoms.Controls
	Public NotInheritable Partial Class StatusControl
		Inherits UserControl
		Public ReadOnly Property StatusList() As ListView
			Get
				Return statusList
			End Get
		End Property

		Public WriteOnly Property HeaderVisibility() As Visibility
			Set(ByVal value As Visibility)
				statusHeader.Visibility = value
			End Set
		End Property

		Public Sub New()
			InitializeComponent()
		End Sub

		Public Sub Log(ByVal nextControl As Control, ByVal s As String)
			ShowMessage(s)
			nextControl.Focus(FocusState.Programmatic)
		End Sub

		Public Sub Log(ByVal s As String)
			ShowMessage(s)
		End Sub

		async Private Sub ShowMessage(ByVal s As String)
			Dim message As String = s ' string.Format("{0}{1}", s, Environment.NewLine);

			If CoreApplication.MainView.CoreWindow.Dispatcher.HasThreadAccess Then
				statusList.Items.Add(message)
				ScrollToBottom(statusList)
			Else
				await CoreApplication.MainView.CoreWindow.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, Sub()
					statusList.Items.Add(message)
					ScrollToBottom(statusList)
				End Sub)
			End If
		End Sub

		async Private Sub ScrollToBottom(ByVal status As ListView)
			Try
				Dim child = VisualTreeHelper.GetChild(status, 0)
				If child IsNot Nothing Then
					Dim childCount As Integer = VisualTreeHelper.GetChildrenCount(child) - 1
					For i As Integer = 0 To childCount
						Dim obj As Object = VisualTreeHelper.GetChild(child, i)
						If Not(TypeOf obj Is ScrollViewer) Then
							Continue For
						End If

						' HACK There's a bug in ScrollViewer.ChangeView, where we must insert a delay before calling 
						' it, otherwise it doesn't work as documented.
						await Task.Delay(100)

						CType(obj, ScrollViewer).ChangeView(0.0f, (CType(obj, ScrollViewer)).ExtentHeight, 1.0f)
						Exit For
					Next i
				End If
			Catch ex As Exception
				Debug.WriteLine("StatusControl.ScrollToBottom: " & ex.ToString())
			End Try
		End Sub
	End Class
End Namespace
