﻿Imports System
Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.Globalization
Imports Windows.Storage
Imports Windows.Storage.Streams
Imports Windows.System
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls
Imports Windows.UI.Xaml.Media.Imaging
Imports Windows.UI.Xaml.Navigation

Namespace SimpleOoms
    Partial Public NotInheritable Class Mainpage
        Inherits Page

#Region "Init"

        Private Const MB As Integer = 1024 * 1024
        Private Const allocationSize As ULong = 10 * 1024 * 1024
        Private consumedMemory As List(Of Byte())

        Public Sub New()
            App.HandleOomExceptions = True
            App.IsActivationDone = True
            InitializeComponent()
            Version.Text = App.VersionString
        End Sub

        Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
            If e.NavigationMode = NavigationMode.New Then
                consumedMemory = New List(Of Byte())()
            End If
        End Sub

        Protected Overrides Sub OnNavigatedFrom(ByVal e As NavigationEventArgs)
            If e.NavigationMode = NavigationMode.Back Then
                consumedMemory.Clear()
                consumedMemory = Nothing
                GC.Collect()
            End If
        End Sub

#End Region


        Private Sub handleOom_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            App.HandleOomExceptions = CBool(handleOom.IsChecked)
        End Sub

        Private Sub failFast_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            App.DoFailFast = CBool(failFast.IsChecked)
        End Sub

        Private Sub scopedAlloc_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            status.Log("allocating byte array.")
            ShowMemoryUsage()

            If App.HandleOomExceptions Then
                Try
                    LoopingMemoryAllocator()
                Catch oomEx As OutOfMemoryException
                    status.Log(oomEx.ToString())
                    If App.DoFailFast Then
                        Environment.FailFast("OutOfMemoryException in DoSomething")
                    End If
                Catch ex As Exception
                    status.Log(ex.Message)
                End Try
            Else
                ' This will eventually throw an OutOfMemoryException, which will invoke the 
                ' App.App_UnhandledException handler.
                LoopingMemoryAllocator()
            End If
        End Sub

        Private Sub LoopingMemoryAllocator()
            Do
                ShowMemoryUsage()
                consumedMemory.Add(New Byte(allocationSize - 1) {})
            Loop
        End Sub

        Private Async Sub unscopedAlloc_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            status.Log("allocating bitmaps.")
            ShowMemoryUsage()

            Dim bitmaps As New List(Of BitmapImage)()
            Dim file As StorageFile = Await StorageFile.GetFileFromApplicationUriAsync(New Uri("ms-appx:///assets/SplashScreen.scale-100.png"))
            Dim stream As IRandomAccessStream = Await file.OpenAsync(FileAccessMode.Read)

            Do
                Try
                    ShowMemoryUsage()
                    stream.Seek(0)
                    Dim bitmap As New BitmapImage()
                    ' This will eventually cause an allocation failure, which will not 
                    ' result in any handleable exceptions to the app.
                    bitmap.SetSource(stream)
                    bitmaps.Add(bitmap)
                Catch oom As OutOfMemoryException
                    status.Log(oom.ToString())
                    Exit Do
                Catch ex As Exception
                    status.Log(ex.Message)
                    Exit Do
                End Try
            Loop
        End Sub

        Private Sub ShowMemoryUsage()
            Try
                Dim usageText As String = String.Format(CultureInfo.CurrentCulture, "usage={0}MB, limit={1}MB", Math.Ceiling(CDbl(MemoryManager.AppMemoryUsage) / MB), Math.Ceiling(CDbl(MemoryManager.AppMemoryUsageLimit) / MB))
                status.Log(usageText)
            Catch ex As Exception
                status.Log(ex.Message)
            End Try
        End Sub


        '/////////////////////////////////////////////// simplified for documentation.
        Private Sub DoSomething()
            Dim buffer As New List(Of Byte())()
            Try
                Do
                    buffer.Add(New Byte(10 * 1024 * 1024 - 1) {})
                Loop
            Catch oomEx As OutOfMemoryException
                ' Handle the exception and recover.
                Debug.WriteLine(oomEx.ToString())
                Environment.FailFast("OutOfMemoryException in DoSomething")
            End Try
        End Sub

        Private Sub DoAnother(ByVal stream As IRandomAccessStream)
            Dim bitmaps As New List(Of BitmapImage)()
            Do
                stream.Seek(0)
                Dim bitmap As New BitmapImage()
                bitmap.SetSource(stream)
                bitmaps.Add(bitmap)
            Loop
        End Sub

    End Class
End Namespace
