﻿Imports System
Imports System.Diagnostics
Imports Windows.ApplicationModel
Imports Windows.ApplicationModel.Activation
Imports Windows.ApplicationModel.Core
Imports Windows.UI.Core
Imports Windows.UI.Popups
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls
Imports Windows.UI.Xaml.Navigation

Namespace TestMemoryApis
	Friend NotInheritable Partial Class App
		Inherits Application

		#Region "Init"

		Public Shared Property IsActivationDone() As Boolean
		Public Shared Property HandleOomExceptions() As Boolean

        Private Shared versionString_Renamed As String = String.Empty
		Public Shared ReadOnly Property VersionString() As String
			Get
				If String.IsNullOrEmpty(versionString) Then
					Dim packageVersion As PackageVersion = Package.Current.Id.Version
					versionString_Renamed = String.Format("{0}.{1}.{2}.{3}", packageVersion.Major, packageVersion.Minor, packageVersion.Build, packageVersion.Revision)
				End If
				Return versionString_Renamed
			End Get
		End Property

		Public Sub New()
			AddHandler UnhandledException, AddressOf App_UnhandledException
			InitializeComponent()
			AddHandler Suspending, AddressOf OnSuspending
		End Sub

		Protected Overrides Sub OnLaunched(ByVal e As LaunchActivatedEventArgs)
			If e.PrelaunchActivated Then
				Debug.WriteLine("LaunchActivatedEventArgs.PrelaunchActivated")
				Return
			End If
			Dim rootFrame As Frame = TryCast(Window.Current.Content, Frame)
			If rootFrame Is Nothing Then
				rootFrame = New Frame()
				rootFrame.Language = Windows.Globalization.ApplicationLanguages.Languages(0)
				AddHandler rootFrame.NavigationFailed, AddressOf OnNavigationFailed
				If e.PreviousExecutionState = ApplicationExecutionState.Terminated Then
				End If

				Window.Current.Content = rootFrame
			End If

			If rootFrame.Content Is Nothing Then
				rootFrame.Navigate(GetType(MainPage), e.Arguments)
			End If
			Window.Current.Activate()
		End Sub

		Private Sub OnNavigationFailed(ByVal sender As Object, ByVal e As NavigationFailedEventArgs)
			Throw New Exception("Failed to load Page " & e.SourcePageType.FullName)
		End Sub

		Private Sub OnSuspending(ByVal sender As Object, ByVal e As SuspendingEventArgs)
            Dim deferral As SuspendingDeferral = e.SuspendingOperation.GetDeferral()
            deferral.Complete()
		End Sub

		#End Region


		' UnhandledException handler will get called for unhandled OOM exceptions, 
		' but not for allocation failures in the XAML framework.
		Private Sub App_UnhandledException(ByVal sender As Object, ByVal e As UnhandledExceptionEventArgs)
			If Debugger.IsAttached Then
				Debugger.Break()
			Else
				If TypeOf e.Exception Is OutOfMemoryException AndAlso (Not HandleOomExceptions) Then
					Return
				End If
				If e.Exception IsNot Nothing AndAlso (Not String.IsNullOrEmpty(e.Exception.Message)) Then
					ShowMessage(e.Exception.Message)
				Else
					ShowMessage("Unhandled Exception")
				End If
				e.Handled = True
			End If
		End Sub

		Friend async Shared Sub ShowMessage(ByVal message As String, Optional ByVal forceShow As Boolean = False)
			If Debugger.IsAttached AndAlso (Not forceShow) Then
				Debug.WriteLine(message)
			Else
				If IsActivationDone Then
					If CoreApplication.MainView.CoreWindow.Dispatcher.HasThreadAccess Then
						Dim dialog As New MessageDialog(message)
						await dialog.ShowAsync()
					Else
						await CoreApplication.MainView.CoreWindow.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, async Sub()
							Dim dialog As New MessageDialog(message)
							await dialog.ShowAsync()
						End Sub)
					End If
				End If
			End If
		End Sub

	End Class
End Namespace
