﻿Imports System
Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.Runtime.InteropServices
Imports Windows.System
Imports Windows.System.Diagnostics
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls

Namespace TestMemoryApis
	Public NotInheritable Partial Class MainPage
		Inherits Page

		#Region "Click handlers"

		Public Sub New()
			InitializeComponent()
		End Sub

		Private Sub tryMemoryIntensiveOperation_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            TryMemoryIntensiveOperationMethod()
        End Sub

        Private Sub moreMemoryIntensiveOperation_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            TryMemoryIntensiveOperation2()
        End Sub

        Private Sub getUsageLevel_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            GetUsageLevelMethod()
        End Sub

        Private Sub registerMemoryCallbacks_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            RegisterMemoryManagerCallbacks()
        End Sub

        Private Sub getDiagnostics_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            GetDiagnosticsMethod()
        End Sub

        Private Sub allocate_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            AllocateMemory()
        End Sub

        Private Sub consume_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            ConsumeMemory()
        End Sub

        Private Sub release_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            ReleaseMemory()
        End Sub

        Private Sub memoryStatus_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            GetMemoryStatus()
        End Sub

        Private Sub pageFile_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            GetPageFileSize()
        End Sub

#End Region


        Private Const SOME_APP_DEFINED_VALUE As ULong = 10 * 1024 * 1024


#Region "MemoryManager"

        Private Sub TryMemoryIntensiveOperationMethod()
            Dim limit As ULong = MemoryManager.AppMemoryUsageLimit
            Dim usage As ULong = MemoryManager.AppMemoryUsage
            Dim headroom As ULong = limit - usage
            If headroom > SOME_APP_DEFINED_VALUE Then
                ' Do memory-intensive operation.
            End If
        End Sub

        Private Sub TryMemoryIntensiveOperation2()
            Dim r As AppMemoryReport = MemoryManager.GetAppMemoryReport()
            Dim headroom As ULong = r.TotalCommitLimit - r.TotalCommitUsage
            If headroom > SOME_APP_DEFINED_VALUE Then
                ' Do memory-intensive operation.
            End If
        End Sub

        Private Sub GetUsageLevelMethod()
            Dim usageLevel As AppMemoryUsageLevel = MemoryManager.AppMemoryUsageLevel
            Select Case usageLevel
                'case AppMemoryUsageLevel.OverLimit:
                    ' Take urgent action to reduce memory usage.
                '    break;
                Case AppMemoryUsageLevel.High
                    ' Take action to reduce memory usage.
                Case AppMemoryUsageLevel.Medium
                    ' Continue to monitor memory usage.
                Case AppMemoryUsageLevel.Low
                    ' Can start memory-intensive operations.
            End Select
        End Sub

#End Region


#Region "MemoryManager Events"

        Public Sub RegisterMemoryManagerCallbacks()
            AddHandler MemoryManager.AppMemoryUsageIncreased, AddressOf OnUsageIncreased
            AddHandler MemoryManager.AppMemoryUsageDecreased, AddressOf OnUsageDecreased
        End Sub

        Private Sub OnUsageIncreased(ByVal sender As Object, ByVal e As Object)
            Dim level As AppMemoryUsageLevel = MemoryManager.AppMemoryUsageLevel
            If level.Equals(AppMemoryUsageLevel.High) Then
                '|| level == AppMemoryUsageLevel.OverLimit){
                ' Take action to reduce memory usage.
            ElseIf level.Equals(AppMemoryUsageLevel.Medium) Then
                ' Continue to monitor memory usage.
            End If
        End Sub

        Private Sub OnUsageDecreased(ByVal sender As Object, ByVal e As Object)
            Dim level As AppMemoryUsageLevel = MemoryManager.AppMemoryUsageLevel
            If level.Equals(AppMemoryUsageLevel.Medium) Then
                ' Back down at an acceptable level.
            ElseIf level.Equals(AppMemoryUsageLevel.Low) Then
                ' Can start memory-intensive operations.
            End If
        End Sub

        Private Sub _RegisterMemoryManagerCallbacks()
            AddHandler MemoryManager.AppMemoryUsageLimitChanging, AddressOf OnMemoryLimitChanging
        End Sub

        Private Sub OnMemoryLimitChanging(ByVal sender As Object, ByVal e As AppMemoryUsageLimitChangingEventArgs)
            If MemoryManager.AppMemoryUsage >= e.NewLimit Then
                ' Take urgent action to reduce memory usage as much as possible.
            End If
            ' Alternatve implementation
            'if (MemoryManager.AppMemoryUsageLevel == 
            '    AppMemoryUsageLevel.OverLimit){
            '    //Take urgent action to reduce memory usage as much as possible.
            '}
        End Sub

#End Region


#Region "Diagnostics"

        Private Sub Trace(ByVal s As String, ByVal u As ULong)
            Debug.WriteLine(s, CLng(CDbl(u) / 1024) \ 1024)
        End Sub

        Private Sub GetDiagnosticsMethod()
            Trace("commit={0:N}", MemoryManager.AppMemoryUsage)
            Dim r1 As ProcessMemoryReport = MemoryManager.GetProcessMemoryReport()
            Trace("private w/s={0:N}", r1.PrivateWorkingSetUsage)
            Trace("total w/s={0:N}", r1.TotalWorkingSetUsage)

            Dim info As ProcessDiagnosticInfo = ProcessDiagnosticInfo.GetForCurrentProcess()
            Dim r2 As ProcessMemoryUsageReport = info.MemoryUsage.GetReport()
            Trace("peak w/s={0:N}", r2.PeakWorkingSetSizeInBytes)
        End Sub

        Private buffer As New List(Of Byte())()

		Private Sub AllocateMemory()
            Dim data(CInt(SOME_APP_DEFINED_VALUE - 1)) As Byte
            buffer.Add(data)
		End Sub

		Private Sub ConsumeMemory()
            Dim data(CInt(SOME_APP_DEFINED_VALUE - 1)) As Byte
            For i As Integer = 0 To data.Length - 1
				data(i) = 1
			Next i
			buffer.Add(data)
		End Sub

		Private Sub ReleaseMemory()
			If buffer.Count > 0 Then
				buffer.RemoveAt(0)
			End If
			GC.Collect()
		End Sub

		#End Region


		#Region "MemoryStatus"

		<StructLayout(LayoutKind.Sequential)>
		Private Class MEMORYSTATUSEX
			Public dwLength As UInteger
			Public dwMemoryLoad As UInteger
			Public ullTotalPhys As ULong
			Public ullAvailPhys As ULong
			Public ullTotalPageFile As ULong
			Public ullAvailPageFile As ULong
			Public ullTotalVirtual As ULong
			Public ullAvailVirtual As ULong
			Public ullAvailExtendedVirtual As ULong
			Public Sub New()
				dwLength = CUInt(Marshal.SizeOf(Of MEMORYSTATUSEX)())
			End Sub
		End Class

		<DllImport("kernelbase.dll", SetLastError := True)>
		Private Shared Function GlobalMemoryStatusEx(<[In](), Out()> ByVal lpBuffer As MEMORYSTATUSEX) As <MarshalAs(UnmanagedType.Bool)> Boolean
		End Function

		Private Sub GetMemoryStatus()
			Dim status As New MEMORYSTATUSEX()
			If GlobalMemoryStatusEx(status) Then
				Trace("TotalPhys={0:N}", status.ullTotalPhys)
				Trace("AvailPhys={0:N}", status.ullAvailPhys)
				Trace("TotalPageFile={0:N}", status.ullTotalPageFile)
				Trace("AvailPageFile={0:N}", status.ullAvailPageFile)
				' ...etc.
			End If
		End Sub

		Private Sub GetPageFileSize()
			Dim status As New MEMORYSTATUSEX()
			If GlobalMemoryStatusEx(status) Then
				Dim pageFile As ULong = status.ullTotalPageFile - status.ullTotalPhys
				Trace("PageFile={0:N}", pageFile)
			End If
		End Sub

		#End Region

	End Class
End Namespace
